﻿using System.Threading;

namespace 委托delegate的使用方法
{
    class Program
    {
        static void Main(string[] args)
        {
            //传统调用
            Calc(calcType.add, 1, 2);
            Calc(calcType.minus, 1, 2);
            Calc(calcType.divide, 1, 2);

            //委托用法
            Calc(add, 1, 2);
            Calc(Minus, 1, 2);
            Calc(Divide, 1, 2);

            //传统的写法（先定义一个方法，然后将方法绑定到线程上，可能这个方法只是临时使用一次）
            Thread t = new Thread(MakeSomeThing);
            t.Start();


            //匿名方法，无需事先定义方法，直接将方法体抽出来
            new Thread(delegate ()
                {
                    //do something
                }).Start();
        }

        //运算类型
        public enum calcType
        {
            add,
            minus,
            divide
        }

        private delegate int CalcDelegate(int x, int y);

        //加法运算
        private static int add(int x, int y)
        {
            return x + y;
        }

        //减法运算
        private static int Minus(int x, int y)
        {
            return x - y;
        }

        //除法运算
        private static int Divide(int x, int y)
        {
            return x / y;
        }

        /// <summary>
        /// 传统的做法（传入运算类型，并调用相应的方法）
        /// </summary>
        private static void Calc(calcType t, int x, int y)
        {
            switch (t)
            {
                case calcType.add:
                    add(x, y);
                    break;

                case calcType.minus:
                    Minus(x, y);
                    break;

                case calcType.divide:
                    Divide(x, y);
                    break;
            }
        }

        /// <summary>
        /// 使用委托的做法（将方法名作为参数传入，直接调用相应的方法）
        /// </summary>
        /// <param name="makeCalc"></param>
        /// <param name="x"></param>
        /// <param name="y"></param>
        private static void Calc(CalcDelegate makeCalc, int x, int y)
        {
            makeCalc(x, y);
        }


        private static void MakeSomeThing()
        {
            //do something
        }
    }
}
